"""Usage:
  edelweiss permissions <dataset-id>
        [--add-user=<email,modifier>]...
        [--remove-user=<email>]...
        [--add-group=<name,modifier>]...
        [--remove-group=<email>]...

Options:
  -h --help      Show this screen

  --add-user=<email,modifier>
        Add a user who can read this dataset.
        The optional modifier can be 'ro' for read-only or 'rw' for 'read-write'.
        Modifier default is ro.
        E.g. --add-user=someone@example.com,rw

  --remove-user=<email>
        Remove a user's access permissions for this dataset

  --add-group=<name,modifier>
        Add a user group who can read this dataset.
        The optional modifier can be 'ro' for read-only or 'rw' for 'read-write'.
        Modifier default is ro.
        E.g. --add-group='my company'

  --remove-group=<name>
        Remove a user group's access permissions for this dataset

"""
from docopt import docopt
import json
from edelweiss_data import DatasetPermissions

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)

    dataset_id = args['<dataset-id>']

    for item in args['--add-user']:
        parts = item.split(',')
        user = DatasetPermissions.User(
                email=parts[0],
                can_write=parts[1] == 'rw' if len(parts) > 1 else False,
                )
        api.add_dataset_user_permission(dataset_id, user)

    for email in args['--remove-user']:
        api.remove_dataset_user_permission(dataset_id, email)

    for item in args['--add-group']:
        parts = item.split(',')
        group = DatasetPermissions.Group(
                name=parts[0],
                can_write=parts[1] == 'rw' if len(parts) > 1 else False,
                )
        api.add_dataset_group_permission(dataset_id, group)

    for group in args['--remove-group']:
        api.remove_dataset_group_permission(dataset_id, group)

    permissions = api.get_dataset_permissions(dataset_id)

    print(json.dumps(permissions.encode(), indent=2 if pretty else None))



"""Usage:
  edelweiss published list [--column=<name,jsonpath>]...
        [--aggregation-filter=<name,value...>]...
        [--condition=<json>] [--search-anywhere=<search>]...
        [--descriptions] [--schemas] [--metadata]
        [--limit=<limit>] [--offset=<offset>] [--no-latest-only]

Options:
  -h --help          Show this screen

  --column=<name,jsonpath>
        A column name and json path describing columns in the dataframe
        Example: --column='compound,$.Assay.Compounds[*]."Compound Name"'
        Specifiy many times for multiple columns
  --aggregation-filter=<name,value...>
        Limits the fetched datasets to ones where column values fall into one of the selected buckets.
        The following example returns datasets where both organ is either liver or kidney AND species
        is either mouse or elephant:
        Example: --aggregation-filter=organ,liver,kidney --aggregation-filter=species,mouse,elephant
  --condition=<json>
        Returns only rows matching the edelweiss query epxression
        Example: --condition='{"fuzzySearch": [{"systemColumn": ["description"]}, "covid"]}'
  --search-anywhere=<search>
        Returns only rows that contain the search term in one of their text-like columns.
        E.g. --search-anywhere=covid
        Equivalent to --condition='{"searchAnywhere": "covid"}'

  --descriptions     Include dataset descriptions in the response
  --schemas          Include dataset schemas in the response
  --metadata         Include dataset metadata in the response
  --limit=<limit>    The number of rows to return [default: 100]
  --offset=<offset>  The list offset, used for incremental fetching [default: 0]
  --no-latest-only   Disables the latest-only feature, which returns only latest version of each dataset.

"""
from docopt import docopt
import json
from edelweiss_data import QueryExpression

def reduce_query_expressions(query_expressions):
    ret = None
    for qe in query_expressions:
        ret = qe if ret is None else ret if qe is None else ret & qe
    return ret

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)
    def parse_filter(f):
        split = f.split(',', 1)
        return (split[0], split[1:])

    search_anywheres = [QueryExpression.search_anywhere(s) for s in args['--search-anywhere']]
    condition = QueryExpression.decode(json.loads(args['--condition'])) if args['--condition'] else None
    condition = reduce_query_expressions(search_anywheres + [condition])

    datasets = api.get_raw_datasets(
        columns=[pair.split(',', 1) for pair in args['--column']],
        condition=condition,
        aggregation_filters=dict(map(parse_filter, args['--aggregation-filter'])),
        include_description=args['--descriptions'],
        include_schema=args['--schemas'], include_metadata=args['--metadata'],
        limit=args['--limit'], latest_only=not args['--no-latest-only'])
    print(json.dumps(datasets, indent=2 if pretty else None))



"""Usage:
  edelweiss published get <id> [<version>] [--simple] [--drop=<key]...

Options:
  -h --help      Show this screen

  --simple       Equivalant to --drop=metadata --drop=description --drop=schema
  --drop=<key>   Do not print the specified key

"""
from docopt import docopt
import json
from edelweiss_data import QueryExpression

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)
    version = int(args['<version>']) if args['<version>'] is not None else None
    dataset = api.get_published_dataset(args['<id>'], version).encode()
    if args['--simple'] or 'metadata' in args['--drop']:
        del dataset['metadata']
    if args['--simple'] or 'schema' in args['--drop']:
        del dataset['schema']
    if args['--simple'] or 'description' in args['--drop']:
        del dataset['description']

    print(json.dumps(dataset, indent=2 if pretty else None))



"""Usage:
  edelweiss inprogress update <id>
            [--name=<name>]
            [--csv=<filename> | --from=<dataset-id>]
            [--metadata=<json> | --metadata-file=<filename.json>]
            [--schema=<json> | --schema-file=<filename.json> | --infer-schema]
            [--description=<text> | --description-file=<filename.md> ]

Update an in-progress dataset

Options:
  -h --help          Show this screen

  --name=<name>
        Name of the dataset.

  --csv=<csv>
        Path to a csv file to upload e.g. --csv=./data.csv
  --from=<dataset-id> 
        Copy data from an existing data source

  --metadata=<json>
        Metadata to attach to this dataset
  --metadata-file=<filename.json>
        File containing metadata to attach to this datast

  --description=<text>
        Description to attach to this dataset.
  --description-file=<filename.md>
        File containing a description of this dataset in markdown or plain text.

  --schema=<json>
        Schema definition of this dataset
  --schema-file=<filename.json>
        File containing the schema definition of this dataset
  --infer-schema
        Let the server infer the schema
"""
from docopt import docopt
import json
import os
from edelweiss_data import Schema

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)

    dataset = api.get_in_progress_dataset(args['<id>'])

    source = api.get_published_dataset(args['--from']) if args['--from'] else None

    if args['--metadata']:
        metadata = json.loads(args['--metadata'])
    elif args['--metadata-file']:
        with open(args['--metadata-file']) as fh:
            metadata = json.load(fh)
    else:
        metadata = None

    if args['--schema']:
        schema = Schema.decode(json.loads(args['--schema']))
    elif args['--schema-file']:
        with open(args['--schema-file']) as fh:
            schema = Schema.decode(json.load(fh))
    else:
        schema = None

    if args['--description']:
        description = args['--description']
    elif args['--description-file']:
        with open(args['--description-file']) as fh:
            description = fh.read()
    else:
        description = None

    name = args['--name'] if args['--name'] else None

    if args['--csv']:
        with open(args['--csv']) as fh:
            dataset.upload_data(fh)

    if metadata or schema or description or source or name:
        dataset.update(
                metadata=metadata,
                schema=schema,
                description=description,
                data_source=source,
                name=name
            )

    if args['--infer-schema']:
        dataset.infer_schema()

    print(json.dumps(dataset.encode(), indent=2 if pretty else None))




"""Usage: edelweiss authenticate [--refresh-token]

Prints an access token for authenticating to the edelweiss API.

Options:
  -h --help          Show this screen
  --refresh-token    Print a refresh token instead of an access token

"""
from docopt import docopt

def run(api, argv):
    args = docopt(__doc__, argv=argv)
    if not api.auth:
        print("Authentication command is incompatible with your authentication options")
        exit(1)
    if args['--refresh-token']:
        print(api.auth.refresh_token)
    else:
        print(api.auth.jwt)



"""Usage:
  edelweiss published new-version <id>
            [--name=<name>]
            [--csv=<filename> | --from=<dataset-id>]
            [--metadata=<json> | --metadata-file=<filename.json>]
            [--schema=<json> | --schema-file=<filename.json> | --infer-schema]
            [--description=<text> | --description-file=<filename.md> ]

Create a new version of a published dataset

Options:
  -h --help          Show this screen

  --name=<name>
        Name for the new version. Defaults to original name.

  --csv=<csv>
        Path to a csv file to upload e.g. --csv=./data.csv
  --from=<dataset-id> 
        Copy data from an existing data source. Defaults to copying from the latest version of the same dataset.

  --metadata=<json>
        Metadata to attach to this dataset
  --metadata-file=<filename.json>
        File containing metadata to attach to this datast

  --description=<text>
        Description to attach to this dataset.
  --description-file=<filename.md>
        File containing a description of this dataset in markdown or plain text.

  --schema=<json>
        Schema definition of this dataset
  --schema-file=<filename.json>
        File containing the schema definition of this dataset
  --infer-schema
        Let the server infer the schema
"""
from docopt import docopt
import json
import os
from edelweiss_data import Schema

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)

    original = api.get_published_dataset(args['<id>'])

    source = api.get_published_dataset(args['--from']) if args['--from'] else original if not args['--csv'] else None

    if args['--metadata']:
        metadata = json.loads(args['--metadata'])
    elif args['--metadata-file']:
        with open(args['--metadata-file']) as fh:
            metadata = json.load(fh)
    else:
        metadata = None

    if args['--schema']:
        schema = Schema.decode(json.loads(args['--schema']))
    elif args['--schema-file']:
        with open(args['--schema-file']) as fh:
            schema = Schema.decode(json.load(fh))
    else:
        schema = None

    if args['--description']:
        description = args['--description']
    elif args['--description-file']:
        with open(args['--description-file']) as fh:
            description = fh.read()
    else:
        description = None

    name = args['--name'] if args['--name'] else None

    dataset = original.new_version()

    try:
        if args['--csv']:
            with open(args['--csv']) as fh:
                dataset.upload_data(fh)

        if metadata or schema or description or source or name:
            dataset.update(
                    metadata=metadata,
                    schema=schema,
                    description=description,
                    data_source=source,
                    name=name
                )

        if args['--infer-schema']:
            dataset.infer_schema()

    except Exception as e:
        try:
            dataset.delete()
        except:
            pass
        raise e

    print(json.dumps(dataset.encode(), indent=2 if pretty else None))




"""Edelweiss Data CLI

A CLI tool for EdelweissData: Convenient publishing of scientific data with proper versioning, rich metadata support and a powerful API.

Usage:
    edelweiss (-h | --help)
    edelweiss [options] authenticate [<args>...]
    edelweiss [options] create [<args>...]
    edelweiss [options] published list [<args>...]
    edelweiss [options] published get [<args>...]
    edelweiss [options] published delete [<args>...]
    edelweiss [options] published new-version [<args>...]
    edelweiss [options] inprogress get [<args>...]
    edelweiss [options] inprogress delete [<args>...]
    edelweiss [options] inprogress update [<args>...]
    edelweiss [options] inprogress publish [<args>...]
    edelweiss [options] permissions [<args>...]

General options:
  -h --help          Show this screen
  --url=URL          The base edelweiss api url [default: https://api.edelweissdata.com]
  --pretty           Use pretty formatting for JSON outputs

Authentication options:
  --token-dir=<dir>  The directory in which to store refresh tokens
  --anonymous        Skip authentication and view only public resources.
  --no-cache-token   Skip caching the authentication token to the `token-dir`
  --refresh-token=<token>
                     Use the provided refresh token instead of the default authentication flow
  --scopes=<scopes>  Any additional authentication scopes to request from the authentication server, comma separated

Help:
    Use the --help option for more details on any sub-command:
    edelweiss published list --help

Examples:
    edelweiss --pretty --anonymous published list --search-anywhere=covid
"""
from docopt import docopt
from edelweiss_data import API
import json

def main():

    args = docopt(__doc__, version="0.1.0", options_first=True)

    if '--help' in args['<args>']:
        # skips setup requests if the subcommand doees not need the api
        api = None
    else:
        api = API(args['--url'])
        if not args['--anonymous']:
            scopes = (args['--scopes'] or '').split(',')
            api.authenticate(cache_jwt=(not args['--no-cache-token']), token_dir=args['--token-dir'],
                    refresh_token=args['--refresh-token'], scopes=scopes, lazy=True)

    if args['authenticate']:
        from . import auth
        return auth.run(api, ['authenticate'] + args['<args>'])
    elif args['published']:
        if args['list']:
            from . import published_list
            return published_list.run(api, ['published', 'list'] + args['<args>'], pretty=args['--pretty'])
        if args['get']:
            from . import published_get
            return published_get.run(api, ['published', 'get'] + args['<args>'], pretty=args['--pretty'])
        if args['delete']:
            from . import published_delete
            return published_delete.run(api, ['published', 'delete'] + args['<args>'], pretty=args['--pretty'])
        if args['new-version']:
            from . import published_newversion
            return published_newversion.run(api, ['published', 'new-version'] + args['<args>'], pretty=args['--pretty'])
    elif args['create']:
            from . import create
            return create.run(api, ['create'] + args['<args>'], pretty=args['--pretty'])
    elif args['inprogress']:
        if args['get']:
            from . import inprogress_get
            return inprogress_get.run(api, ['inprogress', 'get'] + args['<args>'], pretty=args['--pretty'])
        if args['delete']:
            from . import inprogress_delete
            return inprogress_delete.run(api, ['inprogress', 'delete'] + args['<args>'], pretty=args['--pretty'])
        if args['update']:
            from . import inprogress_update
            return inprogress_update.run(api, ['inprogress', 'update'] + args['<args>'], pretty=args['--pretty'])
        if args['publish']:
            from . import inprogress_publish
            return inprogress_publish.run(api, ['inprogress', 'publish'] + args['<args>'], pretty=args['--pretty'])
    elif args['permissions']:
            from . import permissions
            return permissions.run(api, ['permissions'] + args['<args>'], pretty=args['--pretty'])

if __name__ == "__main__":
    main()



"""Usage:
  edelweiss inprogress delete <id>

Options:
  -h --help      Show this screen

"""
from docopt import docopt
import json

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)
    dataset = api.get_in_progress_dataset(args['<id>'])
    dataset.delete()

    print('ok')



"""Usage:
  edelweiss inprogress publish <id> <changelog>

Options:
  -h --help      Show this screen

"""
from docopt import docopt
import json

def run(api, argv, pretty=False):
    args = docopt(__doc__, argv=argv)
    dataset = api.get_in_progress_dataset(args['<id>'])
    dataset = dataset.publish(args['<changelog>'])

    print(json.dumps(dataset.encode(), indent=2 if pretty else None))
